@@ -29,6 +29,8 @@
 #include "progname.h"
 #include "error.h"
 #include "version-etc.h"
+#include "parse-duration.h"
+#include "parse-datetime.h"
 
 #include "oathtool_cmd.h"
 
@@ -62,6 +64,20 @@ usage (int status)
   exit (status);
 }
 
+static time_t
+parse_time (const char *p, const time_t now)
+{
+  struct timespec nowspec = { 0, 0 };
+  struct timespec thenspec;
+
+  nowspec.tv_sec = now;
+
+  if (!parse_datetime (&thenspec, p, &nowspec))
+    return BAD_TIME;
+
+  return thenspec.tv_sec;
+}
+
 #define generate_otp_p(n) ((n) == 1)
 #define validate_otp_p(n) ((n) == 2)
 
@@ -86,11 +102,18 @@ main (int argc, char *argv[])
 
   if (args_info.version_given)
     {
-      const char *p = PACKAGE_NAME;
-      if (strcmp (oath_check_version (NULL), PACKAGE_VERSION) != 0)
-	p = PACKAGE_STRING;
-      version_etc (stdout, "oathtool", p, oath_check_version (NULL),
+      char *p;
+      if (strcmp (oath_check_version ( NULL), OATH_VERSION) != 0)
+	asprintf (&p, "OATH Toolkit liboath.so %s oath.h %s",
+		  oath_check_version (NULL), OATH_VERSION);
+      else if (strcmp (OATH_VERSION, PACKAGE_VERSION) != 0)
+	asprintf (&p, "OATH Toolkit %s",
+		  oath_check_version (NULL), OATH_VERSION);
+      else
+	asprintf (&p, "OATH Toolkit");
+      version_etc (stdout, "oathtool", p, PACKAGE_VERSION,
 		   "Simon Josefsson", (char *) NULL);
+      free (p);
       return EXIT_SUCCESS;
     }
 
@@ -136,10 +159,21 @@ main (int argc, char *argv[])
     error (EXIT_FAILURE, 0, "given one-time password has bad length %d != %d",
 	   args_info.digits_arg, strlen (args_info.inputs[1]));
 
-  if (generate_otp_p (args_info.inputs_num))
+  if (args_info.verbose_flag)
+    {
+      printf ("Hex secret: %s\n", args_info.inputs[0]);
+      printf ("Digits: %d\n", digits);
+      printf ("Window size: %ld\n", window);
+    }
+
+  if (generate_otp_p (args_info.inputs_num) && !args_info.totp_flag)
     {
       size_t iter = 0;
 
+      if (args_info.verbose_flag)
+	printf ("Start counter: 0x%lX (%ld)\n\n",
+		moving_factor, moving_factor);
+
       do
 	{
 	  rc = oath_hotp_generate (secret,
@@ -157,8 +191,71 @@ main (int argc, char *argv[])
 	}
       while (window - iter++ > 0);
     }
+  else if (generate_otp_p (args_info.inputs_num) && args_info.totp_flag)
+    {
+      time_t now = time (NULL);
+      time_t when = parse_time (args_info.now_arg, now);
+      time_t t0 = parse_time (args_info.start_time_arg, now);
+      time_t time_step_size = parse_duration (args_info.time_step_size_arg);
+
+      if (when == BAD_TIME)
+	error (EXIT_FAILURE, 0, "cannot parse time `%s'",
+	       args_info.now_arg);
+
+      if (t0 == BAD_TIME)
+	error (EXIT_FAILURE, 0, "cannot parse time `%s'",
+	       args_info.start_time_arg);
+
+      if (time_step_size == BAD_TIME)
+	error (EXIT_FAILURE, 0, "cannot parse time `%s'",
+	       args_info.time_step_size_arg);
+
+      if (args_info.verbose_flag)
+	{
+	  struct tm tmp;
+	  char outstr[200];
+
+	  if (gmtime_r (&t0, &tmp) == NULL)
+	    error (EXIT_FAILURE, 0, "gmtime_r");
+
+	  if (strftime (outstr, sizeof(outstr),
+			"%Y-%m-%d %H:%M:%S UTC", &tmp) == 0)
+	    error (EXIT_FAILURE, 0, "strftime");
+
+	  printf ("Step size (seconds): %ld\n", time_step_size);
+	  printf ("Start time: %s (%ld)\n", outstr, t0);
+
+	  if (gmtime_r (&when, &tmp) == NULL)
+	    error (EXIT_FAILURE, 0, "gmtime_r");
+
+	  if (strftime (outstr, sizeof(outstr),
+			"%Y-%m-%d %H:%M:%S UTC", &tmp) == 0)
+	    error (EXIT_FAILURE, 0, "strftime");
+
+	  printf ("Time now: %s (%ld)\n", outstr, when);
+	  printf ("Counter: 0x%lX (%ld)\n\n", (when - t0) / time_step_size,
+		  (when - t0) / time_step_size);
+	}
+
+      rc = oath_totp_generate (secret,
+			       secretlen,
+			       when,
+			       time_step_size,
+			       t0,
+			       digits,
+			       otp);
+      if (rc != OATH_OK)
+	error (EXIT_FAILURE, 0,
+	       "generating one-time password failed (%d)", rc);
+
+      printf ("%s\n", otp);
+    }
   else if (validate_otp_p (args_info.inputs_num))
     {
+      if (args_info.verbose_flag)
+	printf ("Start counter: 0x%lX (%ld)\n\n",
+		moving_factor, moving_factor);
+
       rc = oath_hotp_validate (secret,
 			       secretlen,
 			       moving_factor,
